﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Threading.Tasks;
using AzureStorage;
using AzureStorage.Models;
using Microsoft.AspNetCore.Http;
using Microsoft.AspNetCore.Mvc;
using Microsoft.Azure.Cosmos.Table;

namespace AzureTableStorageExample.Controllers
{
    [Route("api/[controller]")]
    [ApiController]
    public class ProductsController : ControllerBase
    {
        INoSQLStorage<Product> _noSQLStorage;
        public ProductsController(INoSQLStorage<Product> noSQLStorage)
        {
            _noSQLStorage = noSQLStorage;
        }
        [HttpGet()]
        public IActionResult GetProducts()
        {
            return Ok(_noSQLStorage.All());
        }
        [HttpGet("{rowKey}")]
        public IActionResult GetProduct(string rowKey)
        {
            return Ok(_noSQLStorage.Query(p => p.RowKey == rowKey));
        }
        [HttpPost()]
        public async Task<IActionResult> Create(Product product)
        {
            Product created = await _noSQLStorage.Add(product);
            return Ok(created);
        }
        [HttpDelete()]
        public async Task<IActionResult> Delete(string rowKey, string partitionKey)
        {
            Product deleted = await _noSQLStorage.Delete(rowKey, partitionKey);
            return Ok(deleted);
        }
        [HttpPut]
        public async Task<IActionResult> Update(Product product)
        {
            try
            {
                product.ETag = "*";
                Product updated = await _noSQLStorage.Update(product);
                return Ok(updated);
            }
            catch (StorageException ex)
            {
                if (ex.RequestInformation.HttpStatusCode == (int)HttpStatusCode.PreconditionFailed)
                    return Ok("PreconditionFailed Exception");
                return Ok(ex.Message);
            }
        }
    }
}
